/****************************************************************************
** Meta object code from reading C++ file 'dmount_global.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../util-dfm/include/dfm-mount/dfm-mount/base/dmount_global.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dmount_global.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8dfmmountE_t {};
} // unnamed namespace

template <> constexpr inline auto dfmmount::qt_create_metaobjectdata<qt_meta_tag_ZN8dfmmountE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dfmmount",
        "DeviceType",
        "kAllDevice",
        "kBlockDevice",
        "kProtocolDevice",
        "kNetDevice",
        "NetworkMountPasswdSaveMode",
        "kNeverSavePasswd",
        "kSaveBeforeLogout",
        "kSavePermanently",
        "MonitorStatus",
        "kMonitoring",
        "kIdle",
        "kNotDefined",
        "Property",
        "kNotInit",
        "kBlockProperty",
        "kBlockConfiguration",
        "kBlockUserspaceMountOptions",
        "kBlockCryptoBackingDevice",
        "kBlockDrive",
        "kBlockIDLabel",
        "kBlockIDType",
        "kBlockIDUsage",
        "kBlockIDUUID",
        "kBlockIDVersion",
        "kBlockDeviceNumber",
        "kBlockPreferredDevice",
        "kBlockID",
        "kBlockSize",
        "kBlockReadOnly",
        "kBlockSymlinks",
        "kBlockHintPartitionable",
        "kBlockHintSystem",
        "kBlockHintIgnore",
        "kBlockHintAuto",
        "kBlockHintName",
        "kBlockHintIconName",
        "kBlockHintSymbolicIconName",
        "kBlockMdRaid",
        "kBlockMdRaidMember",
        "kBlockPropertyEND",
        "kDriveProperty",
        "kDriveConnectionBus",
        "kDriveRemovable",
        "kDriveEjectable",
        "kDriveSeat",
        "kDriveMedia",
        "kDriveMediaCompatibility",
        "kDriveMediaRemovable",
        "kDriveMediaAvailable",
        "kDriveMediaChangeDetected",
        "kDriveTimeDetected",
        "kDriveTimeMediaDetected",
        "kDriveSize",
        "kDriveOptical",
        "kDriveOpticalBlank",
        "kDriveOpticalNumTracks",
        "kDriveOpticalNumAudioTracks",
        "kDriveOpticalNumDataTracks",
        "kDriveOpticalNumSessions",
        "kDriveModel",
        "kDriveRevision",
        "kDriveRotationRate",
        "kDriveSerial",
        "kDriveVender",
        "kDriveWWN",
        "kDriveSortKey",
        "kDriveConfiguration",
        "kDriveID",
        "kDriveCanPowerOff",
        "kDriveSiblingID",
        "kDrivePropertyEND",
        "kFileSystemProperty",
        "kFileSystemMountPoint",
        "kFileSystemPropertyEND",
        "kPartitionProperty",
        "kPartitionNumber",
        "kPartitionType",
        "kPartitionOffset",
        "kPartitionSize",
        "kPartitionFlags",
        "kPartitionName",
        "kPartitionUUID",
        "kPartitionTable",
        "kPartitionIsContainer",
        "kPartitionIsContained",
        "kPartitionPropertyEND",
        "kEncryptedProperty",
        "kEncryptedChildConfiguration",
        "kEncryptedCleartextDevice",
        "kEncryptedHintEncryptionType",
        "kEncryptedMetadataSize",
        "kEncryptedPropertyEnd",
        "kProtoProperty",
        "DeviceError",
        "kNoError",
        "kUnhandledError",
        "kDaemonErrorNotSupportedScheme",
        "kDaemonErrorCannotGenerateMountPath",
        "kDaemonErrorCannotMkdirMountPoint",
        "kUDisksErrorFailed",
        "kUDisksErrorCancelled",
        "kUDisksErrorAlreadyCancelled",
        "kUDisksErrorNotAuthorized",
        "kUDisksErrorNotAuthorizedCanObtain",
        "kUDisksErrorNotAuthorizedDismissed",
        "kUDisksErrorAlreadyMounted",
        "kUDisksErrorNotMounted",
        "kUDisksErrorOptionNotPermitted",
        "kUDisksErrorMountedByOtherUser",
        "kUDisksErrorAlreadyUnmounting",
        "kUDisksErrorNotSupproted",
        "kUDisksErrorTimedOut",
        "kUDisksErrorWouldWakeup",
        "kUDisksErrorDeviceBusy",
        "kUDisksErrorScsiDaemonTransportFailed",
        "kUDisksErrorScsiHostNotFound",
        "kUDisksErrorScsiIDMB",
        "kUDisksErrorScsiLoginFailed",
        "kUDisksErrorScsiLoginAuthFailed",
        "kUDisksErrorScsiLoginFatal",
        "kUDisksErrorScsiLogoutFailed",
        "kUDisksErrorScsiNoFirmware",
        "kUDisksErrorScsiNoObjectsFound",
        "kUDisksErrorScsiNotConnected",
        "kUDisksErrorScsiTransportFailed",
        "kUDisksErrorScsiUnknownDiscoveryType",
        "kUDisksBusySMARTSelfTesting",
        "kUDisksBusyDriveEjecting",
        "kUDisksBusyEncryptedUnlocking",
        "kUDisksBusyEncryptedLocking",
        "kUDisksBusyEncryptedModifying",
        "kUDisksBusyEncryptedResizing",
        "kUDisksBusySwapSpaceStarting",
        "kUDisksBusySwapSpaceStoping",
        "kUDisksBusySwpaSpaceModifying",
        "kUDisksBusyFileSystemUnmounting",
        "kUDisksBusyFileSystemMounting",
        "kUDisksBusyFileSystemModifying",
        "kUDisksBusyFileSystemResizing",
        "kUDisksBusyFormatErasing",
        "kUDisksBusyFormatMkfsing",
        "kUDisksBusyLoopSetuping",
        "kUDisksBusyPartitionModifying",
        "kUDisksBusyPartitionDeleting",
        "kUDisksBusyPartitionCreating",
        "kUDisksBusyCleanuping",
        "kUDisksBusyATASecureErasing",
        "kUDisksBusyATAEnhancedSecureErasing",
        "kUDisksBusyMdRaidStarting",
        "kUDisksBusyMdRaidStoping",
        "kUDisksBusyMdRaidFaultingDevice",
        "kUDisksBusyMdRaidRemovingDevice",
        "kUDisksBusyMdRaidCreating",
        "kGIOError",
        "kGIOErrorFailed",
        "kGIOErrorNotFound",
        "kGIOErrorExists",
        "kGIOErrorIsDirectory",
        "kGIOErrorNotDirertory",
        "kGIOErrorNotEmpty",
        "kGIOErrorNotRegularFile",
        "kGIOErrorNotSymbolicLink",
        "kGIOErrorNotMountableFile",
        "kGIOErrorFilenameTooLong",
        "kGIOErrorInvalidFilename",
        "kGIOErrorTooManyLinks",
        "kGIOErrorNoSpace",
        "kGIOErrorInvalidArgument",
        "kGIOErrorPermissionDenied",
        "kGIOErrorNotSupported",
        "kGIOErrorNotMounted",
        "kGIOErrorAlreadyMounted",
        "kGIOErrorClosed",
        "kGIOErrorCancelled",
        "kGIOErrorPending",
        "kGIOErrorReadONly",
        "kGIOErrorCantCreateBackup",
        "kGIOErrorWrongETAG",
        "kGIOErrorTimedOut",
        "kGIOErrorWouldRecurse",
        "kGIOErrorBusy",
        "kGIOErrorWouldBlock",
        "kGIOErrorHostNotFound",
        "kGIOErrorWouldMerge",
        "kGIOErrorFailedHandled",
        "kGIOErrorTooManyOpenFiles",
        "kGIOErrorNotInitilized",
        "kGIOErrorAddressInUse",
        "kGIOErrorPartialInput",
        "kGIOErrorInvalidData",
        "kGIOErrorDBusError",
        "kGIOErrorHostUnreachable",
        "kGIOErrorNetworkUnreachable",
        "kGIOErrorConnectionRefused",
        "kGIOErrorProxyFailed",
        "kGIOErrorProxyAuthFailed",
        "kGIOErrorProxyNeedAuth",
        "kGIOErrorProxyNotAllowed",
        "kGIOErrorBrokenPipe",
        "kGIOErrorConnectionClosed",
        "kGIOErrorNotConnected",
        "kGIOErrorMessageTooLarge",
        "kGDBusError",
        "kGDBusErrorFailed",
        "kGDBusErrorNoMemory",
        "kGDBusErrorServiceUnknown",
        "kGDBusErrorNameHasNoOwner",
        "kGDBusErrorNoReply",
        "kGDBusErrorIOError",
        "kGDBusErrorBadAddress",
        "kGDBusErrorNotSupported",
        "kGDBusErrorLimitsExceeded",
        "kGDBusErrorAccessDenied",
        "kGDBusErrorAuthFailed",
        "kGDBusErrorNoServer",
        "kGDBusErrorTimeout",
        "kGDBusErrorNoNetwork",
        "kGDBusErrorAddressInUse",
        "kGDBusErrorDisconnected",
        "kGDBusErrorInvalidArgs",
        "kGDBusErrorFileNotFound",
        "kGDBusErrorFileExists",
        "kGDBusErrorUnknownMethod",
        "kGDBusErrorTimedOut",
        "kGDBusErrorMatchRuleNotFound",
        "kGDBusErrorMatchRuleInvalid",
        "kGDBusErrorSpawnExecFailed",
        "kGDBusErrorSpawnForkFailed",
        "kGDBusErrorSpawnChildExited",
        "kGDBusErrorSpawnChildSignaled",
        "kGDBusErrorSpawnFailed",
        "kGDBusErrorSpawnSetupFailed",
        "kGDBusErrorSpawnConfigInvalid",
        "kGDBusErrorSpawnServiceInvalid",
        "kGDBusErrorSpawnServiceNotFound",
        "kGDBusErrorSpawnPermissionsInvalid",
        "kGDBusErrorSpawnFileInvalid",
        "kGDBusErrorSpawnNoMemory",
        "kGDBusErrorUnixProcessIdUnkown",
        "kGDBusErrorInvalidSignature",
        "kGDBusErrorInvalidFileContent",
        "kGDBusErrorSeLinuxSecurityContextUnknown",
        "kGDBusErrorADTAuditDataUnknown",
        "kGDBusErrorObjectPathInUse",
        "kGDBusErrorUnknownObject",
        "kGDBusErrorUnknownInterface",
        "kGDBusErrorUnknownProperty",
        "kGDBusErrorPropertyReadOnly",
        "kUserError",
        "kUserErrorNotMountable",
        "kUserErrorNotEjectable",
        "kUserErrorNoDriver",
        "kUserErrorNotEncryptable",
        "kUserErrorNoPartition",
        "kUserErrorNoBlock",
        "kUserErrorNetworkWrongPasswd",
        "kUserErrorNetworkAnonymousNotAllowed",
        "kUserErrorUserCancelled",
        "kUserErrorTimedOut",
        "kUserErrorAlreadyMounted",
        "kUserErrorNotMounted",
        "kUserErrorNotPoweroffable",
        "kUserErrorFailed",
        "MonitorError",
        "kMonitorAlreadyRegistered",
        "kMonitorNotRegister",
        "PartitionType",
        "kPartitionTypeNotFound",
        "kMbrEmpty",
        "kMbrFAT12Type",
        "kMbrXENIX_root",
        "kMbrXENIX_usr",
        "kMbrFAT16_Less_32M",
        "kMbrExtended",
        "kMbrFAT16Type",
        "kMbrHPFS_NTFS",
        "kMbrAIX",
        "kMbrAIX_bootable",
        "kMbrOS2_Boot_Manager",
        "kMbrWin95_FAT32",
        "kMbrWin95_FAT32_LBA",
        "kMbrWin95_FAT16_LBA",
        "kMbrWin95_Extended_LBA",
        "kMbrOPUS",
        "kMbrHidden_FAT12",
        "kMbrCompaq_diagnostics",
        "kMbrHidden_FAT16_Less_32M",
        "kMbrHidden_FAT16",
        "kMbrHidden_HPFS_or_NTFS",
        "kMbrAST_SmartSleep",
        "kMbrHidden_Win95_FAT32",
        "kMbrHidden_Win95_FAT32_LBA",
        "kMbrHidden_Win95_FAT16_LBA",
        "kMbrNEC_DOS",
        "kMbrPlan9",
        "kMbrPartitionMagic_recovery",
        "kMbrVenix_80286",
        "kMbrPPC_PReP_Boot",
        "kMbrSFS",
        "kMbrQNX4_dot_x",
        "kMbrQNX4_dot_x_2nd_part",
        "kMbrQNX4_dot_x_3rd_part",
        "kMbrOnTrack_DM",
        "kMbrOnTrack_DM6_Aux1",
        "kMbrCP_M",
        "kMbrOnTrack_DM6_Aux3",
        "kMbrOnTrackDM6",
        "kMbrEZ_Drive",
        "kMbrGolden_Bow",
        "kMbrPriam_Edisk",
        "kMbrSpeedStor",
        "kMbrGNU_HURD_or_SysV",
        "kMbrNovell_Netware_286",
        "kMbrNovell_Netware_386",
        "kMbrDiskSecure_Multi_Boot",
        "kMbrPC_IX",
        "kMbrOld_Minix",
        "kMbrMinix_old_Linux",
        "kMbrLinux_swap",
        "kMbrLinux",
        "kMbrOS2_hidden_C_drive",
        "kMbrLinux_extended",
        "kMbrNTFS_volume_set_1",
        "kMbrNTFS_volume_set_2",
        "kMbrLinux_LVM",
        "kMbrAmoeba",
        "kMbrAmoeba_BBT",
        "kMbrBSD_OS",
        "kMbrIBM_Thinkpad_hibernation",
        "kMbrFreeBSD",
        "kMbrOpenBSD",
        "kMbrNeXTSTEP",
        "kMbrNetBSD",
        "kMbrBSDI_fs",
        "kMbrBSDI_swap",
        "kMbrBoot_Wizard_hidden",
        "kMbrDRDOS_sec_FAT12",
        "kMbrDRDOS_sec_FAT16_Less_32M",
        "kMbrDRDOS_sec_FAT16",
        "kMbrDRDOS_sec_extend",
        "kMbrSyrinx",
        "kMbrNon_FS_data",
        "kMbrMultiuser_DOS_extend",
        "kMbrCP_M_CTOS_dot_dot_dot",
        "kMbrDell_Utility",
        "kMbrBootIt",
        "kMbrDOS_access",
        "kMbrDOS_R_O",
        "kMbrSpeedStor_1",
        "kMbrBeOS_fs",
        "kMbrEFI_GPT",
        "kMbrEFI_FAT12_16_32",
        "kMbrLinux_PA_RISC_boot",
        "kMbrSpeedStor_2",
        "kMbrSeppdStor_3",
        "kMbrDOS_secondary",
        "kMbrLinux_raid_autodetect",
        "kMbrLANstep",
        "kMbrBBT",
        "kGptUnusedEntryNA",
        "kGptMBRPartitionSchemeNA",
        "kGptEFISystemPartitionNA",
        "kGptBIOSBootPartitionNA",
        "kGptIntelFastFlashPartitionNA",
        "kGptSonyBootPartitionNA",
        "kGptLenovoBootPartitionNA",
        "kGptMicrosoftReservedPartitionWin",
        "kGptBasicDataPartitionWin",
        "kGptLogicalDiskManagerMetaDataPartitionWin",
        "kGptLogicalDiskManagerDataPartitionWin",
        "kGptWindowsRecoveryEnvironmentWin",
        "kGptIBMGeneralParallelFileSystemPartitionWin",
        "kGptStorageSpacesPartitionWin",
        "kGptStorageReplicaPartitionWin",
        "kGptDataPartitionHPUX",
        "kGptServicePartitionHPUX",
        "kGptLinuxFilesystemDataLinux",
        "kGptRAIDPartitionLinux",
        "kGptRootPartitionX86Linux",
        "kGptRootPartitionX8664Linux",
        "kGptRootPartitionArm32Linux",
        "kGptRootPartitionArm64Linux",
        "kGptBootPartitionLinux",
        "kGptSwapPartitionLinux",
        "kGptLogicalVolumeManagerPartitionLinux",
        "kGptHomePartitionLinux",
        "kGptServerDataPartitionLinux",
        "kGptPlainDMCryptPartitionLinux",
        "kGptLUKSPartitionLinux",
        "kGptReservedLinux",
        "kGptBootPartitionFreeBSD",
        "kGptBSDDisklabelPartitionFreeBSD",
        "kGptSwapPartitionFreeBSD",
        "kGptUnixFileSystemPartitionFreeBSD",
        "kGptVinumVolumeManagerPartitionFreeBSD",
        "kGptZFSPartitionFreeBSD",
        "kGptNandfsPartitionFreeBSD",
        "kGptHierarchialFileSystemPlusPartitionMacOS",
        "kGptAppleAPFSContainerMacOS",
        "kGptAppleUFSContainerMacOS",
        "kGptZFSMacOS",
        "kGptAppleRAIDPartitionMacOS",
        "kGptAppleRAIDPartitionOfflineMacOS",
        "kGptAppleBootPartitionMacOS",
        "kGptAppleLabelMacOS",
        "kGptAppleTVRecoveryPartitionMacOS",
        "kGptAppleCoreStorageContainerMacOS",
        "kGptAppleAPFSPrebootPartitionMacOS",
        "kGptAppleAPFSRecoveryPartitionMacOS",
        "kGptBootPartitionSolaris",
        "kGptRootPartitionSolaris",
        "kGptSwapPartitionSolaris",
        "kGptBackupPartitionSolaris",
        "kGptUsrPartitionSolaris",
        "kGptVarPartitionSolaris",
        "kGptHomePartitionSolaris",
        "kGptAlternateSectorSolaris",
        "kGptReservedPartitionSolaris",
        "kGptSwapPartitionNetBSD",
        "kGptFFSPartitionNetBSD",
        "kGptLFSPartitionNetBSD",
        "kGptRAIDPartitionNetBSD",
        "kGptConcatenatedPartitionNetBSD",
        "kGptEncryptedPartitionNetBSD",
        "kGptKernelChromeOS",
        "kGptRootfsChromeOS",
        "kGptFirmwareChromeOS",
        "kGptFutureUseChromeOS",
        "kGptMiniOSChromeOS",
        "kGptHibernateChromeOS",
        "kGptUsrPartitionCoreOS",
        "kGptResizableRootfsCoreOS",
        "kGptOEMCustomizationsCoreOS",
        "kGptRootFilesystemOnRAIDCoreOS",
        "kGptHaikuBFSHaiku",
        "kGptBootPartitionMidnightBSD",
        "kGptDataPartitionMidnightBSD",
        "kGptSwapPartitionMidnightBSD",
        "kGptUnixFileSystemPartitionMidnightBSD",
        "kGptVinumVolumemanagerPartitionMidnightBSD",
        "kGptZFSPartitionMidnightBSD",
        "kGptJournalCeph",
        "kGptDmCryptJournalCeph",
        "kGptOSDCeph",
        "kGptDmCryptOSDCeph",
        "kGptDiskinCreationCeph",
        "kGptDmCryptDiskinCreationCeph",
        "kGptBlockCeph",
        "kGptBlockDBCeph",
        "kGptBlockWriteAheadlogCeph",
        "kGptLockboxForDmCryptKeysCeph",
        "kGptMultipathOSDCeph",
        "kGptMultipathJournalCeph",
        "kGptMultipathBlockCeph",
        "kGptMultipathBlockDBCeph",
        "kGptMultipathblockwriteAheadogCeph",
        "kGptDmCryptBlockCeph",
        "kGptDmCryptBlockDBCeph",
        "kGptDmCryptBlockWriteAheadlogCeph",
        "kGptDmCryptLUKSjournalCeph",
        "kGptDmCryptLUKSBlockCeph",
        "kGptDmCryptLUKSBlockDBCeph",
        "kGptDmCryptLUKSBlockwriteAheadlogCeph",
        "kGptDmCryptLUKSOSDCeph",
        "kGptDataPartitionOpenBSD",
        "kGptPowerSafeFilesystemQNX",
        "kGptPlan9PartitionPlan9",
        "kGptVmkCoreVMwareESX",
        "kGptVMFSFilesystemPartitionVMwareESX",
        "kGptVMWareReservedVMwareESX",
        "kGptBootloaderAndroidIA",
        "kGptBootloader2AndroidIA",
        "kGptBootAndroidIA",
        "kGptRecoveryAndroidIA",
        "kGptMiscAndroidIA",
        "kGptMetadataAndroidIA",
        "kGptSystemAndroidIA",
        "kGptCacheAndroidIA",
        "kGptDataAndroidIA",
        "kGptPersistentAndroidIA",
        "kGptVendorAndroidIA",
        "kGptConfigAndroidIA",
        "kGptFactoryAndroidIA",
        "kGptFactoryAltAndroidIA",
        "kGptFastbootOrTertiaryAndroidIA",
        "kGptOEMAndroidIA",
        "kGptAndroidMetaAndroid6Arm",
        "kGptAndroidEXTAndroid6Arm",
        "kGptBootONIE",
        "kGptConfigONIE",
        "kGptPRePBootPowerPC",
        "kGptSharedBootloaderConfigurationFreedesktop",
        "kGptBasicDataPartitionAtariTOS",
        "kGptEncryptedDataPartitionVeraCrypt",
        "kGptArcaOSType1OS2",
        "kGptSPDKBlockDeviceSPDK",
        "kGptBareBoxStateBareboxBootloader",
        "kGptUBootEnvironmentUBootBootloader",
        "kGptStatusSoftRAID",
        "kGptScratchSoftRAID",
        "kGptVolumeSoftRAID",
        "kGptCacheSoftRAID",
        "kGptBootloaderFuchsiaStandard",
        "kGptDurablemutableencryptedsystemdataFuchsiaStandard",
        "kGptDurablemutablebootloaderdataFuchsiaStandard",
        "kGptFactoryProvisionedreadOnlysystemdataFuchsiaStandard",
        "kGptFactoryProvisionedreadOnlybootloaderdataFuchsiaStandard",
        "kGptFuchsiaVolumeManagerFuchsiaStandard",
        "kGptVerifiedbootmetadataFuchsiaStandard",
        "kGptZirconbootimageFuchsiaStandard",
        "kGptFuchsiaEspFuchsiaLegacy",
        "kGptFuchsiaSystemFuchsiaLegacy",
        "kGptFuchsiaDataFuchsiaLegacy",
        "kGptFuchsiaInstallFuchsiaLegacy",
        "kGptFuchsiaBlobFuchsiaLegacy",
        "kGptFuchsiaFvmFuchsiaLegacy",
        "kGptZirconbootimageSlotAFuchsiaLegacy",
        "kGptZirconbootimageSlotBFuchsiaLegacy",
        "kGptZirconbootimageSlotRFuchsiaLegacy",
        "kGptSysConfigFuchsiaLegacy",
        "kGptFactoryConfigFuchsiaLegacy",
        "kGptBootloaderFuchsiaLegacy",
        "kGptGuidTestFuchsiaLegacy",
        "kGptVerifiedbootmetadataSlotAFuchsiaLegacy",
        "kGptVerifiedbootmetadataSlotBFuchsiaLegacy",
        "kGptVerifiedbootmetadataSlotRFuchsiaLegacy",
        "kGptMiscFuchsiaLegacy",
        "kGptEmmcBoot1FuchsiaLegacy",
        "kGptEmmcBoot2FuchsiaLegacy"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DeviceType'
        QtMocHelpers::EnumData<DeviceType>(1, 1, QMC::EnumIsScoped).add({
            {    2, DeviceType::kAllDevice },
            {    3, DeviceType::kBlockDevice },
            {    4, DeviceType::kProtocolDevice },
            {    5, DeviceType::kNetDevice },
        }),
        // enum 'NetworkMountPasswdSaveMode'
        QtMocHelpers::EnumData<NetworkMountPasswdSaveMode>(6, 6, QMC::EnumIsScoped).add({
            {    7, NetworkMountPasswdSaveMode::kNeverSavePasswd },
            {    8, NetworkMountPasswdSaveMode::kSaveBeforeLogout },
            {    9, NetworkMountPasswdSaveMode::kSavePermanently },
        }),
        // enum 'MonitorStatus'
        QtMocHelpers::EnumData<MonitorStatus>(10, 10, QMC::EnumIsScoped).add({
            {   11, MonitorStatus::kMonitoring },
            {   12, MonitorStatus::kIdle },
            {   13, MonitorStatus::kNotDefined },
        }),
        // enum 'Property'
        QtMocHelpers::EnumData<Property>(14, 14, QMC::EnumIsScoped).add({
            {   15, Property::kNotInit },
            {   16, Property::kBlockProperty },
            {   17, Property::kBlockConfiguration },
            {   18, Property::kBlockUserspaceMountOptions },
            {   19, Property::kBlockCryptoBackingDevice },
            {    3, Property::kBlockDevice },
            {   20, Property::kBlockDrive },
            {   21, Property::kBlockIDLabel },
            {   22, Property::kBlockIDType },
            {   23, Property::kBlockIDUsage },
            {   24, Property::kBlockIDUUID },
            {   25, Property::kBlockIDVersion },
            {   26, Property::kBlockDeviceNumber },
            {   27, Property::kBlockPreferredDevice },
            {   28, Property::kBlockID },
            {   29, Property::kBlockSize },
            {   30, Property::kBlockReadOnly },
            {   31, Property::kBlockSymlinks },
            {   32, Property::kBlockHintPartitionable },
            {   33, Property::kBlockHintSystem },
            {   34, Property::kBlockHintIgnore },
            {   35, Property::kBlockHintAuto },
            {   36, Property::kBlockHintName },
            {   37, Property::kBlockHintIconName },
            {   38, Property::kBlockHintSymbolicIconName },
            {   39, Property::kBlockMdRaid },
            {   40, Property::kBlockMdRaidMember },
            {   41, Property::kBlockPropertyEND },
            {   42, Property::kDriveProperty },
            {   43, Property::kDriveConnectionBus },
            {   44, Property::kDriveRemovable },
            {   45, Property::kDriveEjectable },
            {   46, Property::kDriveSeat },
            {   47, Property::kDriveMedia },
            {   48, Property::kDriveMediaCompatibility },
            {   49, Property::kDriveMediaRemovable },
            {   50, Property::kDriveMediaAvailable },
            {   51, Property::kDriveMediaChangeDetected },
            {   52, Property::kDriveTimeDetected },
            {   53, Property::kDriveTimeMediaDetected },
            {   54, Property::kDriveSize },
            {   55, Property::kDriveOptical },
            {   56, Property::kDriveOpticalBlank },
            {   57, Property::kDriveOpticalNumTracks },
            {   58, Property::kDriveOpticalNumAudioTracks },
            {   59, Property::kDriveOpticalNumDataTracks },
            {   60, Property::kDriveOpticalNumSessions },
            {   61, Property::kDriveModel },
            {   62, Property::kDriveRevision },
            {   63, Property::kDriveRotationRate },
            {   64, Property::kDriveSerial },
            {   65, Property::kDriveVender },
            {   66, Property::kDriveWWN },
            {   67, Property::kDriveSortKey },
            {   68, Property::kDriveConfiguration },
            {   69, Property::kDriveID },
            {   70, Property::kDriveCanPowerOff },
            {   71, Property::kDriveSiblingID },
            {   72, Property::kDrivePropertyEND },
            {   73, Property::kFileSystemProperty },
            {   74, Property::kFileSystemMountPoint },
            {   75, Property::kFileSystemPropertyEND },
            {   76, Property::kPartitionProperty },
            {   77, Property::kPartitionNumber },
            {   78, Property::kPartitionType },
            {   79, Property::kPartitionOffset },
            {   80, Property::kPartitionSize },
            {   81, Property::kPartitionFlags },
            {   82, Property::kPartitionName },
            {   83, Property::kPartitionUUID },
            {   84, Property::kPartitionTable },
            {   85, Property::kPartitionIsContainer },
            {   86, Property::kPartitionIsContained },
            {   87, Property::kPartitionPropertyEND },
            {   88, Property::kEncryptedProperty },
            {   89, Property::kEncryptedChildConfiguration },
            {   90, Property::kEncryptedCleartextDevice },
            {   91, Property::kEncryptedHintEncryptionType },
            {   92, Property::kEncryptedMetadataSize },
            {   93, Property::kEncryptedPropertyEnd },
            {   94, Property::kProtoProperty },
        }),
        // enum 'DeviceError'
        QtMocHelpers::EnumData<DeviceError>(95, 95, QMC::EnumIsScoped).add({
            {   96, DeviceError::kNoError },
            {   97, DeviceError::kUnhandledError },
            {   98, DeviceError::kDaemonErrorNotSupportedScheme },
            {   99, DeviceError::kDaemonErrorCannotGenerateMountPath },
            {  100, DeviceError::kDaemonErrorCannotMkdirMountPoint },
            {  101, DeviceError::kUDisksErrorFailed },
            {  102, DeviceError::kUDisksErrorCancelled },
            {  103, DeviceError::kUDisksErrorAlreadyCancelled },
            {  104, DeviceError::kUDisksErrorNotAuthorized },
            {  105, DeviceError::kUDisksErrorNotAuthorizedCanObtain },
            {  106, DeviceError::kUDisksErrorNotAuthorizedDismissed },
            {  107, DeviceError::kUDisksErrorAlreadyMounted },
            {  108, DeviceError::kUDisksErrorNotMounted },
            {  109, DeviceError::kUDisksErrorOptionNotPermitted },
            {  110, DeviceError::kUDisksErrorMountedByOtherUser },
            {  111, DeviceError::kUDisksErrorAlreadyUnmounting },
            {  112, DeviceError::kUDisksErrorNotSupproted },
            {  113, DeviceError::kUDisksErrorTimedOut },
            {  114, DeviceError::kUDisksErrorWouldWakeup },
            {  115, DeviceError::kUDisksErrorDeviceBusy },
            {  116, DeviceError::kUDisksErrorScsiDaemonTransportFailed },
            {  117, DeviceError::kUDisksErrorScsiHostNotFound },
            {  118, DeviceError::kUDisksErrorScsiIDMB },
            {  119, DeviceError::kUDisksErrorScsiLoginFailed },
            {  120, DeviceError::kUDisksErrorScsiLoginAuthFailed },
            {  121, DeviceError::kUDisksErrorScsiLoginFatal },
            {  122, DeviceError::kUDisksErrorScsiLogoutFailed },
            {  123, DeviceError::kUDisksErrorScsiNoFirmware },
            {  124, DeviceError::kUDisksErrorScsiNoObjectsFound },
            {  125, DeviceError::kUDisksErrorScsiNotConnected },
            {  126, DeviceError::kUDisksErrorScsiTransportFailed },
            {  127, DeviceError::kUDisksErrorScsiUnknownDiscoveryType },
            {  128, DeviceError::kUDisksBusySMARTSelfTesting },
            {  129, DeviceError::kUDisksBusyDriveEjecting },
            {  130, DeviceError::kUDisksBusyEncryptedUnlocking },
            {  131, DeviceError::kUDisksBusyEncryptedLocking },
            {  132, DeviceError::kUDisksBusyEncryptedModifying },
            {  133, DeviceError::kUDisksBusyEncryptedResizing },
            {  134, DeviceError::kUDisksBusySwapSpaceStarting },
            {  135, DeviceError::kUDisksBusySwapSpaceStoping },
            {  136, DeviceError::kUDisksBusySwpaSpaceModifying },
            {  137, DeviceError::kUDisksBusyFileSystemUnmounting },
            {  138, DeviceError::kUDisksBusyFileSystemMounting },
            {  139, DeviceError::kUDisksBusyFileSystemModifying },
            {  140, DeviceError::kUDisksBusyFileSystemResizing },
            {  141, DeviceError::kUDisksBusyFormatErasing },
            {  142, DeviceError::kUDisksBusyFormatMkfsing },
            {  143, DeviceError::kUDisksBusyLoopSetuping },
            {  144, DeviceError::kUDisksBusyPartitionModifying },
            {  145, DeviceError::kUDisksBusyPartitionDeleting },
            {  146, DeviceError::kUDisksBusyPartitionCreating },
            {  147, DeviceError::kUDisksBusyCleanuping },
            {  148, DeviceError::kUDisksBusyATASecureErasing },
            {  149, DeviceError::kUDisksBusyATAEnhancedSecureErasing },
            {  150, DeviceError::kUDisksBusyMdRaidStarting },
            {  151, DeviceError::kUDisksBusyMdRaidStoping },
            {  152, DeviceError::kUDisksBusyMdRaidFaultingDevice },
            {  153, DeviceError::kUDisksBusyMdRaidRemovingDevice },
            {  154, DeviceError::kUDisksBusyMdRaidCreating },
            {  155, DeviceError::kGIOError },
            {  156, DeviceError::kGIOErrorFailed },
            {  157, DeviceError::kGIOErrorNotFound },
            {  158, DeviceError::kGIOErrorExists },
            {  159, DeviceError::kGIOErrorIsDirectory },
            {  160, DeviceError::kGIOErrorNotDirertory },
            {  161, DeviceError::kGIOErrorNotEmpty },
            {  162, DeviceError::kGIOErrorNotRegularFile },
            {  163, DeviceError::kGIOErrorNotSymbolicLink },
            {  164, DeviceError::kGIOErrorNotMountableFile },
            {  165, DeviceError::kGIOErrorFilenameTooLong },
            {  166, DeviceError::kGIOErrorInvalidFilename },
            {  167, DeviceError::kGIOErrorTooManyLinks },
            {  168, DeviceError::kGIOErrorNoSpace },
            {  169, DeviceError::kGIOErrorInvalidArgument },
            {  170, DeviceError::kGIOErrorPermissionDenied },
            {  171, DeviceError::kGIOErrorNotSupported },
            {  172, DeviceError::kGIOErrorNotMounted },
            {  173, DeviceError::kGIOErrorAlreadyMounted },
            {  174, DeviceError::kGIOErrorClosed },
            {  175, DeviceError::kGIOErrorCancelled },
            {  176, DeviceError::kGIOErrorPending },
            {  177, DeviceError::kGIOErrorReadONly },
            {  178, DeviceError::kGIOErrorCantCreateBackup },
            {  179, DeviceError::kGIOErrorWrongETAG },
            {  180, DeviceError::kGIOErrorTimedOut },
            {  181, DeviceError::kGIOErrorWouldRecurse },
            {  182, DeviceError::kGIOErrorBusy },
            {  183, DeviceError::kGIOErrorWouldBlock },
            {  184, DeviceError::kGIOErrorHostNotFound },
            {  185, DeviceError::kGIOErrorWouldMerge },
            {  186, DeviceError::kGIOErrorFailedHandled },
            {  187, DeviceError::kGIOErrorTooManyOpenFiles },
            {  188, DeviceError::kGIOErrorNotInitilized },
            {  189, DeviceError::kGIOErrorAddressInUse },
            {  190, DeviceError::kGIOErrorPartialInput },
            {  191, DeviceError::kGIOErrorInvalidData },
            {  192, DeviceError::kGIOErrorDBusError },
            {  193, DeviceError::kGIOErrorHostUnreachable },
            {  194, DeviceError::kGIOErrorNetworkUnreachable },
            {  195, DeviceError::kGIOErrorConnectionRefused },
            {  196, DeviceError::kGIOErrorProxyFailed },
            {  197, DeviceError::kGIOErrorProxyAuthFailed },
            {  198, DeviceError::kGIOErrorProxyNeedAuth },
            {  199, DeviceError::kGIOErrorProxyNotAllowed },
            {  200, DeviceError::kGIOErrorBrokenPipe },
            {  201, DeviceError::kGIOErrorConnectionClosed },
            {  202, DeviceError::kGIOErrorNotConnected },
            {  203, DeviceError::kGIOErrorMessageTooLarge },
            {  204, DeviceError::kGDBusError },
            {  205, DeviceError::kGDBusErrorFailed },
            {  206, DeviceError::kGDBusErrorNoMemory },
            {  207, DeviceError::kGDBusErrorServiceUnknown },
            {  208, DeviceError::kGDBusErrorNameHasNoOwner },
            {  209, DeviceError::kGDBusErrorNoReply },
            {  210, DeviceError::kGDBusErrorIOError },
            {  211, DeviceError::kGDBusErrorBadAddress },
            {  212, DeviceError::kGDBusErrorNotSupported },
            {  213, DeviceError::kGDBusErrorLimitsExceeded },
            {  214, DeviceError::kGDBusErrorAccessDenied },
            {  215, DeviceError::kGDBusErrorAuthFailed },
            {  216, DeviceError::kGDBusErrorNoServer },
            {  217, DeviceError::kGDBusErrorTimeout },
            {  218, DeviceError::kGDBusErrorNoNetwork },
            {  219, DeviceError::kGDBusErrorAddressInUse },
            {  220, DeviceError::kGDBusErrorDisconnected },
            {  221, DeviceError::kGDBusErrorInvalidArgs },
            {  222, DeviceError::kGDBusErrorFileNotFound },
            {  223, DeviceError::kGDBusErrorFileExists },
            {  224, DeviceError::kGDBusErrorUnknownMethod },
            {  225, DeviceError::kGDBusErrorTimedOut },
            {  226, DeviceError::kGDBusErrorMatchRuleNotFound },
            {  227, DeviceError::kGDBusErrorMatchRuleInvalid },
            {  228, DeviceError::kGDBusErrorSpawnExecFailed },
            {  229, DeviceError::kGDBusErrorSpawnForkFailed },
            {  230, DeviceError::kGDBusErrorSpawnChildExited },
            {  231, DeviceError::kGDBusErrorSpawnChildSignaled },
            {  232, DeviceError::kGDBusErrorSpawnFailed },
            {  233, DeviceError::kGDBusErrorSpawnSetupFailed },
            {  234, DeviceError::kGDBusErrorSpawnConfigInvalid },
            {  235, DeviceError::kGDBusErrorSpawnServiceInvalid },
            {  236, DeviceError::kGDBusErrorSpawnServiceNotFound },
            {  237, DeviceError::kGDBusErrorSpawnPermissionsInvalid },
            {  238, DeviceError::kGDBusErrorSpawnFileInvalid },
            {  239, DeviceError::kGDBusErrorSpawnNoMemory },
            {  240, DeviceError::kGDBusErrorUnixProcessIdUnkown },
            {  241, DeviceError::kGDBusErrorInvalidSignature },
            {  242, DeviceError::kGDBusErrorInvalidFileContent },
            {  243, DeviceError::kGDBusErrorSeLinuxSecurityContextUnknown },
            {  244, DeviceError::kGDBusErrorADTAuditDataUnknown },
            {  245, DeviceError::kGDBusErrorObjectPathInUse },
            {  246, DeviceError::kGDBusErrorUnknownObject },
            {  247, DeviceError::kGDBusErrorUnknownInterface },
            {  248, DeviceError::kGDBusErrorUnknownProperty },
            {  249, DeviceError::kGDBusErrorPropertyReadOnly },
            {  250, DeviceError::kUserError },
            {  251, DeviceError::kUserErrorNotMountable },
            {  252, DeviceError::kUserErrorNotEjectable },
            {  253, DeviceError::kUserErrorNoDriver },
            {  254, DeviceError::kUserErrorNotEncryptable },
            {  255, DeviceError::kUserErrorNoPartition },
            {  256, DeviceError::kUserErrorNoBlock },
            {  257, DeviceError::kUserErrorNetworkWrongPasswd },
            {  258, DeviceError::kUserErrorNetworkAnonymousNotAllowed },
            {  259, DeviceError::kUserErrorUserCancelled },
            {  260, DeviceError::kUserErrorTimedOut },
            {  261, DeviceError::kUserErrorAlreadyMounted },
            {  262, DeviceError::kUserErrorNotMounted },
            {  263, DeviceError::kUserErrorNotPoweroffable },
            {  264, DeviceError::kUserErrorFailed },
        }),
        // enum 'MonitorError'
        QtMocHelpers::EnumData<MonitorError>(265, 265, QMC::EnumIsScoped).add({
            {   96, MonitorError::kNoError },
            {  266, MonitorError::kMonitorAlreadyRegistered },
            {  267, MonitorError::kMonitorNotRegister },
        }),
        // enum 'PartitionType'
        QtMocHelpers::EnumData<PartitionType>(268, 268, QMC::EnumIsScoped).add({
            {  269, PartitionType::kPartitionTypeNotFound },
            {  270, PartitionType::kMbrEmpty },
            {  271, PartitionType::kMbrFAT12Type },
            {  272, PartitionType::kMbrXENIX_root },
            {  273, PartitionType::kMbrXENIX_usr },
            {  274, PartitionType::kMbrFAT16_Less_32M },
            {  275, PartitionType::kMbrExtended },
            {  276, PartitionType::kMbrFAT16Type },
            {  277, PartitionType::kMbrHPFS_NTFS },
            {  278, PartitionType::kMbrAIX },
            {  279, PartitionType::kMbrAIX_bootable },
            {  280, PartitionType::kMbrOS2_Boot_Manager },
            {  281, PartitionType::kMbrWin95_FAT32 },
            {  282, PartitionType::kMbrWin95_FAT32_LBA },
            {  283, PartitionType::kMbrWin95_FAT16_LBA },
            {  284, PartitionType::kMbrWin95_Extended_LBA },
            {  285, PartitionType::kMbrOPUS },
            {  286, PartitionType::kMbrHidden_FAT12 },
            {  287, PartitionType::kMbrCompaq_diagnostics },
            {  288, PartitionType::kMbrHidden_FAT16_Less_32M },
            {  289, PartitionType::kMbrHidden_FAT16 },
            {  290, PartitionType::kMbrHidden_HPFS_or_NTFS },
            {  291, PartitionType::kMbrAST_SmartSleep },
            {  292, PartitionType::kMbrHidden_Win95_FAT32 },
            {  293, PartitionType::kMbrHidden_Win95_FAT32_LBA },
            {  294, PartitionType::kMbrHidden_Win95_FAT16_LBA },
            {  295, PartitionType::kMbrNEC_DOS },
            {  296, PartitionType::kMbrPlan9 },
            {  297, PartitionType::kMbrPartitionMagic_recovery },
            {  298, PartitionType::kMbrVenix_80286 },
            {  299, PartitionType::kMbrPPC_PReP_Boot },
            {  300, PartitionType::kMbrSFS },
            {  301, PartitionType::kMbrQNX4_dot_x },
            {  302, PartitionType::kMbrQNX4_dot_x_2nd_part },
            {  303, PartitionType::kMbrQNX4_dot_x_3rd_part },
            {  304, PartitionType::kMbrOnTrack_DM },
            {  305, PartitionType::kMbrOnTrack_DM6_Aux1 },
            {  306, PartitionType::kMbrCP_M },
            {  307, PartitionType::kMbrOnTrack_DM6_Aux3 },
            {  308, PartitionType::kMbrOnTrackDM6 },
            {  309, PartitionType::kMbrEZ_Drive },
            {  310, PartitionType::kMbrGolden_Bow },
            {  311, PartitionType::kMbrPriam_Edisk },
            {  312, PartitionType::kMbrSpeedStor },
            {  313, PartitionType::kMbrGNU_HURD_or_SysV },
            {  314, PartitionType::kMbrNovell_Netware_286 },
            {  315, PartitionType::kMbrNovell_Netware_386 },
            {  316, PartitionType::kMbrDiskSecure_Multi_Boot },
            {  317, PartitionType::kMbrPC_IX },
            {  318, PartitionType::kMbrOld_Minix },
            {  319, PartitionType::kMbrMinix_old_Linux },
            {  320, PartitionType::kMbrLinux_swap },
            {  321, PartitionType::kMbrLinux },
            {  322, PartitionType::kMbrOS2_hidden_C_drive },
            {  323, PartitionType::kMbrLinux_extended },
            {  324, PartitionType::kMbrNTFS_volume_set_1 },
            {  325, PartitionType::kMbrNTFS_volume_set_2 },
            {  326, PartitionType::kMbrLinux_LVM },
            {  327, PartitionType::kMbrAmoeba },
            {  328, PartitionType::kMbrAmoeba_BBT },
            {  329, PartitionType::kMbrBSD_OS },
            {  330, PartitionType::kMbrIBM_Thinkpad_hibernation },
            {  331, PartitionType::kMbrFreeBSD },
            {  332, PartitionType::kMbrOpenBSD },
            {  333, PartitionType::kMbrNeXTSTEP },
            {  334, PartitionType::kMbrNetBSD },
            {  335, PartitionType::kMbrBSDI_fs },
            {  336, PartitionType::kMbrBSDI_swap },
            {  337, PartitionType::kMbrBoot_Wizard_hidden },
            {  338, PartitionType::kMbrDRDOS_sec_FAT12 },
            {  339, PartitionType::kMbrDRDOS_sec_FAT16_Less_32M },
            {  340, PartitionType::kMbrDRDOS_sec_FAT16 },
            {  341, PartitionType::kMbrDRDOS_sec_extend },
            {  342, PartitionType::kMbrSyrinx },
            {  343, PartitionType::kMbrNon_FS_data },
            {  344, PartitionType::kMbrMultiuser_DOS_extend },
            {  345, PartitionType::kMbrCP_M_CTOS_dot_dot_dot },
            {  346, PartitionType::kMbrDell_Utility },
            {  347, PartitionType::kMbrBootIt },
            {  348, PartitionType::kMbrDOS_access },
            {  349, PartitionType::kMbrDOS_R_O },
            {  350, PartitionType::kMbrSpeedStor_1 },
            {  351, PartitionType::kMbrBeOS_fs },
            {  352, PartitionType::kMbrEFI_GPT },
            {  353, PartitionType::kMbrEFI_FAT12_16_32 },
            {  354, PartitionType::kMbrLinux_PA_RISC_boot },
            {  355, PartitionType::kMbrSpeedStor_2 },
            {  356, PartitionType::kMbrSeppdStor_3 },
            {  357, PartitionType::kMbrDOS_secondary },
            {  358, PartitionType::kMbrLinux_raid_autodetect },
            {  359, PartitionType::kMbrLANstep },
            {  360, PartitionType::kMbrBBT },
            {  361, PartitionType::kGptUnusedEntryNA },
            {  362, PartitionType::kGptMBRPartitionSchemeNA },
            {  363, PartitionType::kGptEFISystemPartitionNA },
            {  364, PartitionType::kGptBIOSBootPartitionNA },
            {  365, PartitionType::kGptIntelFastFlashPartitionNA },
            {  366, PartitionType::kGptSonyBootPartitionNA },
            {  367, PartitionType::kGptLenovoBootPartitionNA },
            {  368, PartitionType::kGptMicrosoftReservedPartitionWin },
            {  369, PartitionType::kGptBasicDataPartitionWin },
            {  370, PartitionType::kGptLogicalDiskManagerMetaDataPartitionWin },
            {  371, PartitionType::kGptLogicalDiskManagerDataPartitionWin },
            {  372, PartitionType::kGptWindowsRecoveryEnvironmentWin },
            {  373, PartitionType::kGptIBMGeneralParallelFileSystemPartitionWin },
            {  374, PartitionType::kGptStorageSpacesPartitionWin },
            {  375, PartitionType::kGptStorageReplicaPartitionWin },
            {  376, PartitionType::kGptDataPartitionHPUX },
            {  377, PartitionType::kGptServicePartitionHPUX },
            {  378, PartitionType::kGptLinuxFilesystemDataLinux },
            {  379, PartitionType::kGptRAIDPartitionLinux },
            {  380, PartitionType::kGptRootPartitionX86Linux },
            {  381, PartitionType::kGptRootPartitionX8664Linux },
            {  382, PartitionType::kGptRootPartitionArm32Linux },
            {  383, PartitionType::kGptRootPartitionArm64Linux },
            {  384, PartitionType::kGptBootPartitionLinux },
            {  385, PartitionType::kGptSwapPartitionLinux },
            {  386, PartitionType::kGptLogicalVolumeManagerPartitionLinux },
            {  387, PartitionType::kGptHomePartitionLinux },
            {  388, PartitionType::kGptServerDataPartitionLinux },
            {  389, PartitionType::kGptPlainDMCryptPartitionLinux },
            {  390, PartitionType::kGptLUKSPartitionLinux },
            {  391, PartitionType::kGptReservedLinux },
            {  392, PartitionType::kGptBootPartitionFreeBSD },
            {  393, PartitionType::kGptBSDDisklabelPartitionFreeBSD },
            {  394, PartitionType::kGptSwapPartitionFreeBSD },
            {  395, PartitionType::kGptUnixFileSystemPartitionFreeBSD },
            {  396, PartitionType::kGptVinumVolumeManagerPartitionFreeBSD },
            {  397, PartitionType::kGptZFSPartitionFreeBSD },
            {  398, PartitionType::kGptNandfsPartitionFreeBSD },
            {  399, PartitionType::kGptHierarchialFileSystemPlusPartitionMacOS },
            {  400, PartitionType::kGptAppleAPFSContainerMacOS },
            {  401, PartitionType::kGptAppleUFSContainerMacOS },
            {  402, PartitionType::kGptZFSMacOS },
            {  403, PartitionType::kGptAppleRAIDPartitionMacOS },
            {  404, PartitionType::kGptAppleRAIDPartitionOfflineMacOS },
            {  405, PartitionType::kGptAppleBootPartitionMacOS },
            {  406, PartitionType::kGptAppleLabelMacOS },
            {  407, PartitionType::kGptAppleTVRecoveryPartitionMacOS },
            {  408, PartitionType::kGptAppleCoreStorageContainerMacOS },
            {  409, PartitionType::kGptAppleAPFSPrebootPartitionMacOS },
            {  410, PartitionType::kGptAppleAPFSRecoveryPartitionMacOS },
            {  411, PartitionType::kGptBootPartitionSolaris },
            {  412, PartitionType::kGptRootPartitionSolaris },
            {  413, PartitionType::kGptSwapPartitionSolaris },
            {  414, PartitionType::kGptBackupPartitionSolaris },
            {  415, PartitionType::kGptUsrPartitionSolaris },
            {  416, PartitionType::kGptVarPartitionSolaris },
            {  417, PartitionType::kGptHomePartitionSolaris },
            {  418, PartitionType::kGptAlternateSectorSolaris },
            {  419, PartitionType::kGptReservedPartitionSolaris },
            {  420, PartitionType::kGptSwapPartitionNetBSD },
            {  421, PartitionType::kGptFFSPartitionNetBSD },
            {  422, PartitionType::kGptLFSPartitionNetBSD },
            {  423, PartitionType::kGptRAIDPartitionNetBSD },
            {  424, PartitionType::kGptConcatenatedPartitionNetBSD },
            {  425, PartitionType::kGptEncryptedPartitionNetBSD },
            {  426, PartitionType::kGptKernelChromeOS },
            {  427, PartitionType::kGptRootfsChromeOS },
            {  428, PartitionType::kGptFirmwareChromeOS },
            {  429, PartitionType::kGptFutureUseChromeOS },
            {  430, PartitionType::kGptMiniOSChromeOS },
            {  431, PartitionType::kGptHibernateChromeOS },
            {  432, PartitionType::kGptUsrPartitionCoreOS },
            {  433, PartitionType::kGptResizableRootfsCoreOS },
            {  434, PartitionType::kGptOEMCustomizationsCoreOS },
            {  435, PartitionType::kGptRootFilesystemOnRAIDCoreOS },
            {  436, PartitionType::kGptHaikuBFSHaiku },
            {  437, PartitionType::kGptBootPartitionMidnightBSD },
            {  438, PartitionType::kGptDataPartitionMidnightBSD },
            {  439, PartitionType::kGptSwapPartitionMidnightBSD },
            {  440, PartitionType::kGptUnixFileSystemPartitionMidnightBSD },
            {  441, PartitionType::kGptVinumVolumemanagerPartitionMidnightBSD },
            {  442, PartitionType::kGptZFSPartitionMidnightBSD },
            {  443, PartitionType::kGptJournalCeph },
            {  444, PartitionType::kGptDmCryptJournalCeph },
            {  445, PartitionType::kGptOSDCeph },
            {  446, PartitionType::kGptDmCryptOSDCeph },
            {  447, PartitionType::kGptDiskinCreationCeph },
            {  448, PartitionType::kGptDmCryptDiskinCreationCeph },
            {  449, PartitionType::kGptBlockCeph },
            {  450, PartitionType::kGptBlockDBCeph },
            {  451, PartitionType::kGptBlockWriteAheadlogCeph },
            {  452, PartitionType::kGptLockboxForDmCryptKeysCeph },
            {  453, PartitionType::kGptMultipathOSDCeph },
            {  454, PartitionType::kGptMultipathJournalCeph },
            {  455, PartitionType::kGptMultipathBlockCeph },
            {  456, PartitionType::kGptMultipathBlockDBCeph },
            {  457, PartitionType::kGptMultipathblockwriteAheadogCeph },
            {  458, PartitionType::kGptDmCryptBlockCeph },
            {  459, PartitionType::kGptDmCryptBlockDBCeph },
            {  460, PartitionType::kGptDmCryptBlockWriteAheadlogCeph },
            {  461, PartitionType::kGptDmCryptLUKSjournalCeph },
            {  462, PartitionType::kGptDmCryptLUKSBlockCeph },
            {  463, PartitionType::kGptDmCryptLUKSBlockDBCeph },
            {  464, PartitionType::kGptDmCryptLUKSBlockwriteAheadlogCeph },
            {  465, PartitionType::kGptDmCryptLUKSOSDCeph },
            {  466, PartitionType::kGptDataPartitionOpenBSD },
            {  467, PartitionType::kGptPowerSafeFilesystemQNX },
            {  468, PartitionType::kGptPlan9PartitionPlan9 },
            {  469, PartitionType::kGptVmkCoreVMwareESX },
            {  470, PartitionType::kGptVMFSFilesystemPartitionVMwareESX },
            {  471, PartitionType::kGptVMWareReservedVMwareESX },
            {  472, PartitionType::kGptBootloaderAndroidIA },
            {  473, PartitionType::kGptBootloader2AndroidIA },
            {  474, PartitionType::kGptBootAndroidIA },
            {  475, PartitionType::kGptRecoveryAndroidIA },
            {  476, PartitionType::kGptMiscAndroidIA },
            {  477, PartitionType::kGptMetadataAndroidIA },
            {  478, PartitionType::kGptSystemAndroidIA },
            {  479, PartitionType::kGptCacheAndroidIA },
            {  480, PartitionType::kGptDataAndroidIA },
            {  481, PartitionType::kGptPersistentAndroidIA },
            {  482, PartitionType::kGptVendorAndroidIA },
            {  483, PartitionType::kGptConfigAndroidIA },
            {  484, PartitionType::kGptFactoryAndroidIA },
            {  485, PartitionType::kGptFactoryAltAndroidIA },
            {  486, PartitionType::kGptFastbootOrTertiaryAndroidIA },
            {  487, PartitionType::kGptOEMAndroidIA },
            {  488, PartitionType::kGptAndroidMetaAndroid6Arm },
            {  489, PartitionType::kGptAndroidEXTAndroid6Arm },
            {  490, PartitionType::kGptBootONIE },
            {  491, PartitionType::kGptConfigONIE },
            {  492, PartitionType::kGptPRePBootPowerPC },
            {  493, PartitionType::kGptSharedBootloaderConfigurationFreedesktop },
            {  494, PartitionType::kGptBasicDataPartitionAtariTOS },
            {  495, PartitionType::kGptEncryptedDataPartitionVeraCrypt },
            {  496, PartitionType::kGptArcaOSType1OS2 },
            {  497, PartitionType::kGptSPDKBlockDeviceSPDK },
            {  498, PartitionType::kGptBareBoxStateBareboxBootloader },
            {  499, PartitionType::kGptUBootEnvironmentUBootBootloader },
            {  500, PartitionType::kGptStatusSoftRAID },
            {  501, PartitionType::kGptScratchSoftRAID },
            {  502, PartitionType::kGptVolumeSoftRAID },
            {  503, PartitionType::kGptCacheSoftRAID },
            {  504, PartitionType::kGptBootloaderFuchsiaStandard },
            {  505, PartitionType::kGptDurablemutableencryptedsystemdataFuchsiaStandard },
            {  506, PartitionType::kGptDurablemutablebootloaderdataFuchsiaStandard },
            {  507, PartitionType::kGptFactoryProvisionedreadOnlysystemdataFuchsiaStandard },
            {  508, PartitionType::kGptFactoryProvisionedreadOnlybootloaderdataFuchsiaStandard },
            {  509, PartitionType::kGptFuchsiaVolumeManagerFuchsiaStandard },
            {  510, PartitionType::kGptVerifiedbootmetadataFuchsiaStandard },
            {  511, PartitionType::kGptZirconbootimageFuchsiaStandard },
            {  512, PartitionType::kGptFuchsiaEspFuchsiaLegacy },
            {  513, PartitionType::kGptFuchsiaSystemFuchsiaLegacy },
            {  514, PartitionType::kGptFuchsiaDataFuchsiaLegacy },
            {  515, PartitionType::kGptFuchsiaInstallFuchsiaLegacy },
            {  516, PartitionType::kGptFuchsiaBlobFuchsiaLegacy },
            {  517, PartitionType::kGptFuchsiaFvmFuchsiaLegacy },
            {  518, PartitionType::kGptZirconbootimageSlotAFuchsiaLegacy },
            {  519, PartitionType::kGptZirconbootimageSlotBFuchsiaLegacy },
            {  520, PartitionType::kGptZirconbootimageSlotRFuchsiaLegacy },
            {  521, PartitionType::kGptSysConfigFuchsiaLegacy },
            {  522, PartitionType::kGptFactoryConfigFuchsiaLegacy },
            {  523, PartitionType::kGptBootloaderFuchsiaLegacy },
            {  524, PartitionType::kGptGuidTestFuchsiaLegacy },
            {  525, PartitionType::kGptVerifiedbootmetadataSlotAFuchsiaLegacy },
            {  526, PartitionType::kGptVerifiedbootmetadataSlotBFuchsiaLegacy },
            {  527, PartitionType::kGptVerifiedbootmetadataSlotRFuchsiaLegacy },
            {  528, PartitionType::kGptMiscFuchsiaLegacy },
            {  529, PartitionType::kGptEmmcBoot1FuchsiaLegacy },
            {  530, PartitionType::kGptEmmcBoot2FuchsiaLegacy },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN8dfmmountE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN8dfmmountE =
    dfmmount::qt_create_metaobjectdata<qt_meta_tag_ZN8dfmmountE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN8dfmmountE =
    qt_staticMetaObjectContent_ZN8dfmmountE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN8dfmmountE =
    qt_staticMetaObjectContent_ZN8dfmmountE.relocatingData;

Q_CONSTINIT const QMetaObject dfmmount::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN8dfmmountE.stringdata,
    qt_staticMetaObjectStaticContent_ZN8dfmmountE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN8dfmmountE.metaTypes,
    nullptr
} };

QT_WARNING_POP
