// This file is autogenerated. Changes will be overwritten.
#include "TGCD52HF2M/moc_dopticaldiscmanager.cpp"
#include "TGCD52HF2M/moc_dpacketwritingcontroller.cpp"
#include "PGVKHOT7RA/moc_dudfburnengine.cpp"
#include "PGVKHOT7RA/moc_dxorrisoengine.cpp"
